'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getLogs = getLogs;

var _winston = require('winston');

var _winston2 = _interopRequireDefault(_winston);

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

var _authorisation = require('./authorisation');

var authorisation = _interopRequireWildcard(_authorisation);

var _utils = require('../utils');

var utils = _interopRequireWildcard(_utils);

var _util = require('util');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const levels = {
  debug: 1,
  info: 2,
  warn: 3,
  error: 4
};

async function getLogs(ctx) {
  // Only admins can view server logs
  if (!authorisation.inGroup('admin', ctx.authenticated)) {
    utils.logAndSetResponse(ctx, 403, `User ${ctx.authenticated.email} is not an admin, API access to getLogs denied.`, 'info');
    return;
  }

  let { query } = ctx.request || {};
  if (query == null) {
    query = {};
  }

  // default to info level logs
  if (query.level == null) {
    query.level = 'info';
  }

  const options = {
    from: query.from != null ? (0, _moment2.default)(query.from).toDate() : (0, _moment2.default)().subtract(5, 'minutes').toDate(),
    until: (0, _moment2.default)(query.until || undefined).toDate(),
    order: 'asc',
    start: parseInt(query.start, 10) || 0,
    limit: 100000 // limit: 0 doesn't work :/
  };

  let results = await (0, _util.promisify)(_winston2.default.query.bind(_winston2.default))(options);
  results = results.mongodb;

  if (query.level != null) {
    results = results.filter(item => levels[item.level] >= levels[query.level]);
  }

  if (query.limit != null) {
    results.splice(query.limit, results.length - query.limit);
  }

  ctx.body = results;
  ctx.status = 200;
}
//# sourceMappingURL=logs.js.map