'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.config = exports.appRoot = undefined;

var _nconf = require('nconf');

var _nconf2 = _interopRequireDefault(_nconf);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _winston = require('winston');

var _winston2 = _interopRequireDefault(_winston);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const appRoot = exports.appRoot = _path2.default.resolve(__dirname, '../..');

/*
 * Define the default constructor
 */
function Config() {
  // Get the argument-value to use
  _nconf2.default.argv().env('_');

  // don't read NODE:ENV from nconf as it could be overwritten by any env var starting with 'NODE_'
  const environment = process.env.NODE_ENV;

  const conf = _nconf2.default.get('conf');

  // Load the configuration-values
  // user specified config override
  if (conf) {
    if (!_fs2.default.existsSync(conf)) {
      _winston2.default.warn(`Invalid config path ${conf}`);
    }
    _nconf2.default.file('customConfigOverride', conf);
  }

  // environment override
  if (environment) {
    const envPath = `${appRoot}/config/${environment}.json`;
    if (!_fs2.default.existsSync(envPath)) {
      _winston2.default.warn(`No config found for env ${environment} at path ${envPath}`);
    }
    _nconf2.default.file('environmentOverride', `${appRoot}/config/${environment}.json`);
  }

  // load the default config file
  _nconf2.default.file('default', `${appRoot}/config/default.json`);

  // Return the result
}

/*
 * This function return the value that was set in the key-value store
 */
Config.prototype.get = key => _nconf2.default.get(key);

/*
 * This function constructs a new instanse of this class
 */
const config = exports.config = new Config();
//# sourceMappingURL=config.js.map