'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.AuditMetaModel = exports.AuditModel = undefined;

var _mongoose = require('mongoose');

var _config = require('../config');

const codeTypeDef = {
  code: String,
  displayName: String,
  codeSystemName: String
};

const syslogDef = {
  prival: Number,
  facilityID: Number,
  severityID: Number,
  facility: String,
  severity: String,
  type: { type: String },
  time: Date,
  host: String,
  appName: String,
  pid: String,
  msgID: String
};

const ActiveParticipantDef = {
  userID: String,
  alternativeUserID: String,
  userIsRequestor: String,
  networkAccessPointID: String,
  networkAccessPointTypeCode: String,
  roleIDCode: codeTypeDef
};

const ParticipantObjectIdentificationDef = {
  participantObjectID: String,
  participantObjectTypeCode: String,
  participantObjectTypeCodeRole: String,
  participantObjectIDTypeCode: codeTypeDef,
  participantObjectQuery: String,
  participantObjectDetail: {
    type: { type: String },
    value: String
  }
};

const AuditRecordSchema = new _mongoose.Schema({
  rawMessage: String,
  syslog: syslogDef,
  eventIdentification: {
    eventDateTime: {
      type: Date, required: true, default: Date.now, index: true
    },
    eventOutcomeIndicator: String,
    eventActionCode: String,
    eventID: codeTypeDef,
    eventTypeCode: codeTypeDef
  },
  activeParticipant: [ActiveParticipantDef],
  auditSourceIdentification: {
    auditSourceID: String,
    auditEnterpriseSiteID: String,
    auditSourceTypeCode: codeTypeDef
  },
  participantObjectIdentification: [ParticipantObjectIdentificationDef]
});

// keeps track of unique codes for various fields found in the audits collection
const AuditMetaRecordSchema = new _mongoose.Schema({
  eventType: [codeTypeDef],
  eventID: [codeTypeDef],
  activeParticipantRoleID: [codeTypeDef],
  participantObjectIDTypeCode: [codeTypeDef],
  auditSourceID: [String]
}, {
  collection: 'auditMeta'
});

const AuditModel = exports.AuditModel = _config.connectionATNA.model('Audit', AuditRecordSchema);
const AuditMetaModel = exports.AuditMetaModel = _config.connectionATNA.model('AuditMeta', AuditMetaRecordSchema);
//# sourceMappingURL=audits.js.map