'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.VisualizerModel = exports.VisualizerModelAPI = undefined;

var _mongoose = require('mongoose');

var _config = require('../config');

var _events = require('./events');

var events = _interopRequireWildcard(_events);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

const EventLinkDef = {
  eventType: {
    type: String, enum: events.eventTypes
  },
  eventName: String,
  display: String
};

const MediatorLinkDef = {
  mediator: String, // mediator URN
  name: String,
  display: String
};

const VisualizerSchema = new _mongoose.Schema({
  name: {
    type: String, required: true, unique: true
  },
  components: [EventLinkDef],
  channels: [EventLinkDef],
  mediators: [MediatorLinkDef],
  color: {
    inactive: {
      type: String, default: '#cccccc'
    },
    active: {
      type: String, default: '#4cae4c'
    },
    error: {
      type: String, default: '#d43f3a'
    },
    text: {
      type: String, default: '#000000'
    }
  },
  size: {
    responsive: {
      type: Boolean, default: true
    },
    width: {
      type: Number, default: 1000
    },
    height: {
      type: Number, default: 400
    },
    padding: {
      type: Number, default: 20
    }
  },
  time: {
    updatePeriod: {
      type: Number, default: 200
    },
    minDisplayPeriod: {
      type: Number, default: 500
    },
    maxSpeed: {
      type: Number, default: 5
    },
    maxTimeout: {
      type: Number, default: 5000
    }
  }
});

// Compile schema into Model
const VisualizerModelAPI = exports.VisualizerModelAPI = _config.connectionAPI.model('Visualizer', VisualizerSchema);
const VisualizerModel = exports.VisualizerModel = _config.connectionDefault.model('Visualizer', VisualizerSchema);
//# sourceMappingURL=visualizer.js.map