'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.agendaGlobal = undefined;
exports.registerPollingChannel = registerPollingChannel;
exports.removePollingChannel = removePollingChannel;
exports.setupAgenda = setupAgenda;

var _request = require('request');

var _request2 = _interopRequireDefault(_request);

var _winston = require('winston');

var _winston2 = _interopRequireDefault(_winston);

var _channels = require('./model/channels');

var Channels = _interopRequireWildcard(_channels);

var _config = require('./config');

var _utils = require('./utils');

var utils = _interopRequireWildcard(_utils);

var _util = require('util');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const { ChannelModel } = Channels;
_config.config.polling = _config.config.get('polling');

let agendaGlobal = exports.agendaGlobal = null;

function registerPollingChannel(channel, callback) {
  _winston2.default.info(`Registering polling channel: ${channel._id}`);
  if (!channel.pollingSchedule) {
    return callback(new Error('no polling schedule set on this channel'));
  }

  return exports.agendaGlobal.cancel({ name: `polling-job-${channel._id}` }, err => {
    if (err) {
      return callback(err);
    }
    exports.agendaGlobal.define(`polling-job-${channel._id}`, (job, done) => {
      _winston2.default.info(`Polling channel ${channel._id}`);

      const options = {
        url: `http://${_config.config.polling.host}:${_config.config.polling.pollingPort}/trigger`,
        headers: {
          'channel-id': channel._id,
          'X-OpenHIM-LastRunAt': job.attrs.lastRunAt
        }
      };

      return (0, _request2.default)(options, () => done());
    });

    exports.agendaGlobal.every(channel.pollingSchedule, `polling-job-${channel._id}`, null, { timezone: utils.serverTimezone() });

    return callback(null);
  });
}

function removePollingChannel(channel, callback) {
  _winston2.default.info(`Removing polling schedule for channel: ${channel._id}`);
  return exports.agendaGlobal.cancel({ name: `polling-job-${channel._id}` }, err => {
    if (err) {
      return callback(err);
    }
    return callback(null);
  });
}

function setupAgenda(agenda, callback) {
  _winston2.default.info('Starting polling server...');
  const registerPollingChannelPromise = (0, _util.promisify)(registerPollingChannel);
  exports.agendaGlobal = agendaGlobal = agenda;
  return ChannelModel.find({ type: 'polling' }, (err, channels) => {
    if (err) {
      return err;
    }

    const promises = [];
    for (const channel of Array.from(channels)) {
      if (Channels.isChannelEnabled(channel)) {
        promises.push(registerPollingChannelPromise(channel));
      }
    }

    return Promise.all(promises).then(callback);
  });
}
//# sourceMappingURL=polling.js.map